(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('date-fns'), require('@vue/composition-api')) :
  typeof define === 'function' && define.amd ? define(['date-fns', '@vue/composition-api'], factory) :
  (global = global || self, global.vueAirbnbCalendarSelection = factory(global.dateFns, global.compositionApi));
}(this, (function (dateFns, compositionApi) { 'use strict';

  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
  }

  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;

    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

    return arr2;
  }

  function _createForOfIteratorHelper(o, allowArrayLike) {
    var it;

    if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
      if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
        if (it) o = it;
        var i = 0;

        var F = function () {};

        return {
          s: F,
          n: function () {
            if (i >= o.length) return {
              done: true
            };
            return {
              done: false,
              value: o[i++]
            };
          },
          e: function (e) {
            throw e;
          },
          f: F
        };
      }

      throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
    }

    var normalCompletion = true,
        didErr = false,
        err;
    return {
      s: function () {
        it = o[Symbol.iterator]();
      },
      n: function () {
        var step = it.next();
        normalCompletion = step.done;
        return step;
      },
      e: function (e) {
        didErr = true;
        err = e;
      },
      f: function () {
        try {
          if (!normalCompletion && it.return != null) it.return();
        } finally {
          if (didErr) throw err;
        }
      }
    };
  }

  function unwrapExports (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  function createCommonjsModule(fn, module) {
  	return module = { exports: {} }, fn(module, module.exports), module.exports;
  }

  var styles = createCommonjsModule(function (module, exports) {

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.cssVar = void 0;

  var lodash_1 = require("lodash");

  exports.cssVar = function (name) {
    return '--dp-' + lodash_1.kebabCase(name);
  };
  });

  unwrapExports(styles);
  var styles_1 = styles.cssVar;

  var normalizeRanges = function normalizeRanges(ranges) {
    if (!ranges) {
      return [];
    }

    return (ranges instanceof Function ? ranges() : ranges).map(function (r) {
      return {
        start: dateFns.startOfDay(r.start),
        end: dateFns.startOfDay(r.end)
      };
    });
  };
  var intervalOverlapsWith = function intervalOverlapsWith(interval, ranges) {
    if (!ranges) {
      return;
    }

    ranges = normalizeRanges(ranges);

    if (!(ranges.length && interval.start && interval.end)) {
      return false;
    }

    var selectionInterval = orderedInterval(interval.start, interval.end);

    var _iterator = _createForOfIteratorHelper(ranges),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var range = _step.value;

        if (dateFns.areIntervalsOverlapping(selectionInterval, range)) {
          return true;
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return false;
  };
  var orderedInterval = function orderedInterval(date1, date2) {
    var start = dateFns.isBefore(date1, date2) ? date1 : date2;
    var end = date1 === start ? date2 : date1;
    return {
      start: start,
      end: end
    };
  };
  var findRangesOfItem = function findRangesOfItem(item, ranges) {
    return findRangesOfDate(item.date, ranges);
  };
  var findRangesOfDate = function findRangesOfDate(date, ranges) {
    if (!ranges) {
      return [];
    }

    ranges = normalizeRanges(ranges);
    return ranges.filter(function (range) {
      return dateFns.isWithinInterval(date, range);
    });
  };

  var makeDateInfo = function makeDateInfo() {
    return {
      reservation: false,
      checkIn: false,
      checkOut: false
    };
  };

  var getInfo = function getInfo(date, analysis) {
    return analysis[date.getTime()] || (analysis[date.getTime()] = makeDateInfo());
  };
  var analyzeOptions = function analyzeOptions(options) {
    if (!options) {
      return {};
    }

    var analysis = {};
    var reservationOptions = options.reservations || {};
    var reservationRanges = normalizeRanges(reservationOptions.ranges);

    var _iterator = _createForOfIteratorHelper(reservationRanges),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var range = _step.value;
        var dates = dateFns.eachDayOfInterval(range);

        for (var i = 0; i < dates.length; ++i) {
          var date = dates[i];
          var dateInfo = getInfo(date, analysis);
          dateInfo.reservation = true;

          if (i === 0) {
            dateInfo.checkIn = true;
          }

          if (i === dates.length - 1) {
            dateInfo.checkOut = true;
          }
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return analysis;
  };

  var hasReservation = function hasReservation(item, analysis) {
    return !!(analysis && analysis[item.date.getTime()] && analysis[item.date.getTime()].reservation);
  };
  var validateCheckInOut = function validateCheckInOut(selection, item, analysis, reservationRanges, selectableWeekday, minDays, options) {
    var resOptions = options.reservations;

    if (!resOptions) {
      return true;
    }

    if (!minDays) {
      return true;
    }

    var maxGap = resOptions.maxGap ? resOptions.maxGap instanceof Function ? resOptions.maxGap({
      selection: selection
    }) : resOptions.maxGap : 0; // validate selection start

    if (!(selection.from || selection.to) || selection.from && selection.to) {
      // shift dates depending on whether check-in & check-out days can overlap
      var shift = resOptions.allowCheckInOutOverlap ? 0 : 1;
      var isCheckIn = getInfo(dateFns.addDays(item.date, shift), analysis).checkIn;
      var isCheckOut = getInfo(dateFns.subDays(item.date, shift), analysis).checkOut; // always allow check-in and check-out days to be selected if resOptions.allowGapFills is true

      if (resOptions.allowGapFills && isCheckOut != isCheckIn) {
        return true;
      }

      if (isCheckOut && !intervalOverlapsWith({
        start: item.date,
        end: dateFns.addDays(item.date, minDays)
      }, reservationRanges)) {
        return true;
      }

      if (isCheckIn && !intervalOverlapsWith({
        start: dateFns.subDays(item.date, minDays),
        end: item.date
      }, reservationRanges)) {
        return true;
      }

      if (!selectableWeekday) {
        return false;
      } // allow check-in with a maximum gap to another reservation of maxGap


      var _iterator = _createForOfIteratorHelper(reservationRanges),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var range = _step.value;

          if (dateFns.isBefore(item.date, range.start)) {
            // check if item.date is in the gap zone before a reservation and minDays is would be fulfilled
            if (dateFns.differenceInDays(range.start, item.date) <= maxGap + shift && !intervalOverlapsWith({
              start: dateFns.subDays(item.date, minDays),
              end: item.date
            }, reservationRanges)) {
              return true;
            }
          } else if (dateFns.isAfter(item.date, range.end)) {
            // check if item.date is in the gap zone after a reservation and minDays is would be fulfilled
            if (dateFns.differenceInDays(item.date, range.end) <= maxGap + shift && !intervalOverlapsWith({
              start: item.date,
              end: dateFns.addDays(item.date, minDays)
            }, reservationRanges)) {
              return true;
            }
          }
        } // check if minDays would be fulfilled, if selection.start is not a checkOut and allowGapFills is false

      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      if (!resOptions.allowGapFills && !isCheckOut && (intervalOverlapsWith({
        start: dateFns.subDays(item.date, minDays + shift),
        end: item.date
      }, reservationRanges) || intervalOverlapsWith({
        start: item.date,
        end: dateFns.addDays(item.date, minDays + shift)
      }, reservationRanges))) {
        return false;
      }
    } // validate selection end


    if (selection.from && !selection.to) {
      // shift dates depending on whether check-in & check-out days can overlap
      var _shift = resOptions.allowCheckInOutOverlap ? 0 : 1;

      var interval = orderedInterval(selection.from.date, item.date);
      var days = dateFns.differenceInDays(interval.end, interval.start); // check if selection would wrap a reservation

      if (intervalOverlapsWith(interval, reservationRanges)) {
        return false;
      } // allow gap fills


      if (days < minDays && resOptions.allowGapFills) {
        if (dateFns.isAfter(item.date, selection.from.date) && getInfo(dateFns.addDays(item.date, _shift), analysis).checkIn // && getInfo(subDays(selection.from.date, shift), analysis).checkOut
        ) {
            return true;
          } else if (dateFns.isBefore(item.date, selection.from.date) && getInfo(dateFns.subDays(item.date, _shift), analysis).checkOut // && getInfo(addDays(selection.from.date, shift), analysis).checkIn
        ) {
            return true;
          }
      }

      var dateInfo = getInfo(item.date, analysis);

      if (!selection.from) {
        if (dateInfo.checkOut) {
          return true;
        }
      }

      if (days < minDays) {
        return false;
      } // always allow seletion to connect to check-in & check-out days


      if (dateInfo.checkIn != dateInfo.checkOut) {
        return true;
      }

      if (!selectableWeekday) {
        return false;
      }

      var _iterator2 = _createForOfIteratorHelper(reservationRanges),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _range = _step2.value;

          // check if item.date is in the gap zone before or after a reservation and minDays is would be fulfilled
          if (dateFns.isBefore(item.date, _range.start) && dateFns.differenceInDays(_range.start, item.date) <= maxGap + _shift || dateFns.isAfter(item.date, _range.end) && dateFns.differenceInDays(item.date, _range.end) <= maxGap + _shift) {
            return true;
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      _shift = resOptions.allowCheckInOutOverlap ? 0 : 2;

      if (dateFns.isBefore(item.date, selection.from.date)) {
        if (intervalOverlapsWith({
          start: dateFns.subDays(interval.start, minDays + _shift),
          end: interval.start
        }, reservationRanges)) {
          return false;
        }
      }

      if (dateFns.isAfter(item.date, selection.from.date)) {
        if (intervalOverlapsWith({
          start: interval.end,
          end: dateFns.addDays(interval.end, minDays + _shift)
        }, reservationRanges)) {
          return false;
        }
      }
    }

    return true;
  };

  var containers = {};
  var selectItem = function selectItem(item, selection) {
    if (selection.value.from && selection.value.to) {
      selection.value = {
        from: item,
        to: null
      };
      return;
    }

    if (!selection.value.from) {
      selection.value.from = item;
      return;
    }

    var from = selection.value.from;
    var to = item;

    if (dateFns.isBefore(to.date, from.date)) {
      to = from;
      from = item;
    }

    selection.value = {
      from: from,
      to: to
    };
  };
  var clear = function clear(selection) {
    selection.value.from = null;
    selection.value.to = null;
  };

  var isBeforeMinDate = function isBeforeMinDate(item, minDate) {
    return !!minDate && dateFns.isBefore(item.date, minDate);
  };

  var isAfterMaxDate = function isAfterMaxDate(item, maxDate) {
    return !!maxDate && dateFns.isAfter(item.date, maxDate);
  };

  var isSelectableWeekday = function isSelectableWeekday(item, weekdays) {
    if (!weekdays || !weekdays.length) {
      return true;
    }

    if (typeof weekdays[0] === 'number') {
      return weekdays.indexOf(dateFns.getDay(item.date)) > -1;
    }

    var _iterator = _createForOfIteratorHelper(weekdays),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var config = _step.value;

        if (!dateFns.isWithinInterval(item.date, config.range)) {
          continue;
        }

        return config.weekdays.indexOf(dateFns.getDay(item.date)) > -1;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return true;
  };

  function useSelection(id, options) {
    var _ref = containers[id] || (containers[id] = {
      selection: compositionApi.ref({
        from: null,
        to: null
      }),
      hoverItem: compositionApi.ref(null),
      analysis: compositionApi.ref({})
    }),
        selection = _ref.selection,
        hoverItem = _ref.hoverItem,
        analysis = _ref.analysis;

    var isHovered = function isHovered(item) {
      return !!hoverItem.value && item.date.getTime() === hoverItem.value.date.getTime();
    };

    var isSelected = function isSelected(item) {
      return !!(selection.value.from && selection.value.from.date.getTime() === item.date.getTime() || selection.value.to && selection.value.to.date.getTime() === item.date.getTime());
    };

    var isWithinSelection = function isWithinSelection(item) {
      if (!item.isCurrentMonth) {
        return false;
      }

      var hoverSelection = {
        from: selection.value.from || hoverItem.value,
        to: selection.value.to || hoverItem.value
      };

      if (hoverSelection.from && hoverSelection.to && dateFns.isAfter(hoverSelection.from.date, hoverSelection.to.date)) {
        var from = hoverSelection.from;
        var to = hoverSelection.to;
        hoverSelection.from = to;
        hoverSelection.to = from;
      }

      return !!(hoverSelection.from && hoverSelection.to && dateFns.isWithinInterval(item.date, {
        start: hoverSelection.from.date,
        end: hoverSelection.to.date
      }));
    };

    var isSelectable = function isSelectable(item, analysis, options) {
      if (!item.isCurrentMonth) {
        return false;
      }

      var resOptions = options.reservations || {};
      var minDays = resOptions.minDays instanceof Function ? resOptions.minDays({
        selection: selection.value
      }) : resOptions.minDays || 0;
      var reservationRanges = normalizeRanges(resOptions.ranges || []);
      var selectableRanges = findRangesOfItem(item, options.selectableRanges);
      var selectableWeekday = isSelectableWeekday(item, options.selectableWeekdays);
      var beforeMinDate = isBeforeMinDate(item, options.minDate);
      var afterMaxDate = isAfterMaxDate(item, options.maxDate);
      var validCheckInOut = validateCheckInOut(selection.value, item, analysis, reservationRanges, selectableWeekday, minDays, options);
      var defaultValue = !(options.selectableRanges && !selectableRanges.length || !validCheckInOut || beforeMinDate || afterMaxDate);

      if (options.selectable) {
        return options.selectable(item, {
          selectableRanges: selectableRanges,
          selectableWeekday: selectableWeekday,
          reservationRanges: reservationRanges,
          beforeMinDate: beforeMinDate,
          afterMaxDate: afterMaxDate,
          defaultValue: defaultValue,
          selection: selection.value
        });
      }

      return defaultValue;
    };

    var stopHandles = compositionApi.ref([]);
    stopHandles.value.push(compositionApi.watch(options, function (options) {
      analysis.value = analyzeOptions(options);
    }, {
      immediate: true
    }));

    var destroy = function destroy() {
      var _iterator2 = _createForOfIteratorHelper(stopHandles.value),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var stop = _step2.value;
          stop();
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
    };

    return {
      selection: selection,
      selectItem: selectItem,
      hoverItem: hoverItem,
      analysis: analysis,
      isHovered: isHovered,
      clear: clear,
      isSelectable: isSelectable,
      isSelected: isSelected,
      isWithinSelection: isWithinSelection,
      analyzeOptions: analyzeOptions,
      hasReservation: hasReservation,
      isBeforeMinDate: isBeforeMinDate,
      isAfterMaxDate: isAfterMaxDate,
      stopHandles: stopHandles,
      destroy: destroy
    };
  }

  var renderDay = function renderDay(h, path) {
    return h('div', {
      staticClass: 'AirbnbCalendarItem__checkInOut'
    }, [h('svg', {
      class: 'AirbnbCalendarItem__checkInOut-svg',
      attrs: {
        viewBox: '0 0 100 100',
        preserveAspectRatio: 'none',
        'shape-rendering': 'geometricPrecision'
      }
    }, [h('path', {
      attrs: {
        d: path
      }
    })])]);
  };
  var renderCheckInDay = function renderCheckInDay(h) {
    return renderDay(h, 'M-1 101 L 101 101 L 101 -1 L -1 101');
  };
  var renderCheckOutDay = function renderCheckOutDay(h) {
    return renderDay(h, 'M-1 -1 L 101 -1 L -1 101 L -1 -1');
  };

  var index = (function () {
    var opt = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var options = compositionApi.isRef(opt) ? opt : compositionApi.ref(opt);
    return function (rootContext, _ref, onDestroy) {
      var installRootPlugin = _ref.installRootPlugin,
          installCalendarItemPlugin = _ref.installCalendarItemPlugin;

      var _useSelection = useSelection(rootContext.id, options),
          selection = _useSelection.selection,
          hoverItem = _useSelection.hoverItem,
          isHovered = _useSelection.isHovered,
          selectItem = _useSelection.selectItem,
          isSelectable = _useSelection.isSelectable,
          isSelected = _useSelection.isSelected,
          isWithinSelection = _useSelection.isWithinSelection,
          isBeforeMinDate = _useSelection.isBeforeMinDate,
          isAfterMaxDate = _useSelection.isAfterMaxDate,
          analysis = _useSelection.analysis,
          hasReservation = _useSelection.hasReservation,
          stopHandles = _useSelection.stopHandles,
          destroy = _useSelection.destroy,
          clearSelection = _useSelection.clear;

      compositionApi.onMounted(function () {
        stopHandles.value.push(compositionApi.watch(function () {
          return options.value.colors;
        }, function (colors) {
          for (var state in colors) {
            var stateColors = colors[state];

            for (var key in stateColors) {
              var color = stateColors[key];
              document.documentElement.style.setProperty(styles_1("sel-color-".concat(state, "-").concat(key)), color);
            }
          }
        }, {
          immediate: true
        }));
      });
      onDestroy(function () {
        destroy(); // Doesn't work anymore ?
        // for (const state in options.value.colors) {
        //   const stateColors = options.value.colors[state as keyof Options<F>['colors']] as CalendarItemColors
        //   for (const key in stateColors) {
        //     document.documentElement.style.removeProperty(cssVar(`sel-color-${state}-${key}`))
        //   }
        // }
      });
      stopHandles.value.push(compositionApi.watch(selection, function (sel) {
        if ((sel.from || sel.to) && options.value.onSelect) {
          // @ts-ignore
          var from = sel.from ? typeof options.value.dateFormat === 'undefined' ? sel.from.date : dateFns.format(sel.from.date, options.value.dateFormat) : null; // @ts-ignore

          var to = sel.to ? typeof options.value.dateFormat === 'undefined' ? sel.to.date : dateFns.format(sel.to.date, options.value.dateFormat) : null;
          options.value.onSelect.apply(options, [{
            from: from,
            to: to
          }]);
        }
      }, {
        deep: true,
        immediate: true
      }));
      installRootPlugin({
        on: {
          keyup: function keyup(el, ev) {
            if (ev.key === 'Escape') {
              clearSelection(selection);
            }
          }
        }
      });
      installCalendarItemPlugin({
        classes: function classes(item) {
          var classes = [];

          if (!item.isCurrentMonth) {
            return classes;
          }

          var selectable = isSelectable(item, analysis.value, options.value);
          var blocked = hasReservation(item, analysis.value);
          var hovered = isHovered(item);
          var withinSelection = isWithinSelection(item);
          var selected = isSelected(item);
          var highlighted = options.value.highlight ? options.value.highlight(item, {
            hovered: hovered,
            selectable: selectable,
            selected: selected,
            withinSelection: withinSelection,
            blocked: blocked
          }) : false;
          var blockedStartDay = false;
          var blockedEndDay = false;

          if (options.value.reservations && options.value.reservations.allowCheckInOutOverlap) {
            var blockedRanges = findRangesOfItem(item, options.value.reservations ? options.value.reservations.ranges : undefined);

            var _iterator = _createForOfIteratorHelper(blockedRanges),
                _step;

            try {
              for (_iterator.s(); !(_step = _iterator.n()).done;) {
                var range = _step.value;

                if (dateFns.isSameDay(range.start, item.date)) {
                  blockedStartDay = true;
                }

                if (dateFns.isSameDay(range.end, item.date)) {
                  blockedEndDay = true;
                }
              }
            } catch (err) {
              _iterator.e(err);
            } finally {
              _iterator.f();
            }
          }

          if (selectable) {
            classes.push('is-selectable');
          } else {
            classes.push('is-unselectable');
          }

          if (hovered) {
            classes.push('is-hovered');
          }

          if (withinSelection) {
            classes.push('is-within-selection');
          }

          if (selected) {
            classes.push('is-selected');
          }

          if (highlighted) {
            classes.push('is-highlighted');
          }

          if (blocked) {
            classes.push('is-blocked');
          }

          if (!(blockedStartDay && blockedEndDay)) {
            if (blockedStartDay) {
              classes.push('is-blocked-start-day');
            }

            if (blockedEndDay) {
              classes.push('is-blocked-end-day');
            }
          }

          return classes;
        },
        styles: function styles(item, classes, ctx) {
          var styles = {};

          if (!options.value.css) {
            return styles;
          }

          if (options.value.css.base) {
            styles = Object.assign({}, styles, options.value.css.base);
          }

          if (options.value.css.selectable && classes.indexOf('is-selectable') > -1) {
            styles = Object.assign({}, styles, options.value.css.selectable);
          }

          if (options.value.css.unselectable && classes.indexOf('is-unselectable') > -1) {
            styles = Object.assign({}, styles, options.value.css.unselectable);
          }

          if (options.value.css.hovered && classes.indexOf('is-hovered') > -1) {
            styles = Object.assign({}, styles, options.value.css.hovered);
          }

          if (options.value.css.withinSelection && classes.indexOf('is-within-selection') > -1) {
            styles = Object.assign({}, styles, options.value.css.withinSelection);
          }

          if (options.value.css.selected && classes.indexOf('is-selected') > -1) {
            styles = Object.assign({}, styles, options.value.css.selected);
          }

          if (options.value.css.highlighted && classes.indexOf('is-highlighted') > -1) {
            styles = Object.assign({}, styles, options.value.css.highlighted);
          }

          if (options.value.css.blocked && classes.indexOf('is-blocked') > -1) {
            styles = Object.assign({}, styles, options.value.css.blocked);
          }

          return styles;
        },
        on: {
          click: function click(item) {
            if (isSelectable(item, analysis.value, options.value)) {
              selectItem(item, selection);
            }
          },
          mouseenter: function mouseenter(item) {
            if (!item.isCurrentMonth || isBeforeMinDate(item, options.value.minDate) || isAfterMaxDate(item, options.value.maxDate)) {
              hoverItem.value = null;
              return;
            }

            hoverItem.value = item;
          },
          mouseleave: function mouseleave(item) {
            if (hoverItem.value === item) {
              hoverItem.value = null;
            }
          }
        },
        calendarItemRenderFn: function calendarItemRenderFn(h, prev, _ref2) {
          var classes = _ref2.classes;

          if (!(options.value.reservations && options.value.reservations.allowCheckInOutOverlap)) {
            return prev;
          }

          if (classes.indexOf('is-hovered') > -1 || classes.indexOf('is-selected') > -1) {
            return prev;
          }

          if (classes.indexOf('is-blocked-start-day') > -1) {
            prev.children.unshift(renderCheckInDay(h));
          }

          if (classes.indexOf('is-blocked-end-day') > -1) {
            prev.children.unshift(renderCheckOutDay(h));
          }

          return prev;
        }
      });
    };
  });

  return index;

})));
